#!/usr/bin/env python
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2011 Yiannis Tsagatakis (talos) jtsagata@jmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

###################### DO NOT TOUCH THIS (HEAD TO THE SECOND PART) ######################

import os
import sys

try:
    import DistUtilsExtra.auto
except ImportError:
    print >> sys.stderr, 'To build thistle-tales you need https://launchpad.net/python-distutils-extra'
    sys.exit(1)
assert DistUtilsExtra.auto.__version__ >= '2.18', 'needs DistUtilsExtra.auto >= 2.18'

#
# The C Extension
#

from distutils.core import setup, Extension
import commands
import subprocess
def invoke_pkg_config (cmd):
    result  = commands.getoutput(cmd)
    return [part for part in result.strip().split (' ') if part]

c_hacks_extension = Extension (
    name               = 'gtktree._impl.c_hacks',
    sources            = [os.path.join ('thistle_tales','gtktree', '_impl', 'c_hacks.c')],
    extra_compile_args = invoke_pkg_config("pkg-config --cflags gtk+-2.0 pygobject-2.0"),
    extra_link_args    = invoke_pkg_config("pkg-config --libs gtk+-2.0 pygobject-2.0")
)

def update_data_path(prefix, oldvalue=None):

    try:
        fin = file('thistle_tales/thistle_talesconfig.py', 'r')
        fout = file(fin.name + '.new', 'w')

        for line in fin:            
            fields = line.split(' = ') # Separate variable from value
            if fields[0] == '__thistle_tales_data_directory__':
                # update to prefix, store oldvalue
                if not oldvalue:
                    oldvalue = fields[1]
                    line = "%s = '%s'\n" % (fields[0], prefix)
                else: # restore oldvalue
                    line = "%s = %s" % (fields[0], oldvalue)
            fout.write(line)

        fout.flush()
        fout.close()
        fin.close()
        os.rename(fout.name, fin.name)
    except (OSError, IOError), e:
        print ("ERROR: Can't find thistle_tales/thistle_talesconfig.py")
        sys.exit(1)
    return oldvalue


def update_desktop_file(datadir):

    try:
        fin = file('thistle-tales.desktop.in', 'r')
        fout = file(fin.name + '.new', 'w')

        for line in fin:            
            if 'Icon=' in line:
                line = "Icon=%s\n" % (datadir + 'media/thistle-tales.svg')
            fout.write(line)
        fout.flush()
        fout.close()
        fin.close()
        os.rename(fout.name, fin.name)
    except (OSError, IOError), e:
        print ("ERROR: Can't find thistle-tales.desktop.in")
        sys.exit(1)


class InstallAndUpdateDataDirectory(DistUtilsExtra.auto.install_auto):
    def run(self):
        previous_value = update_data_path(self.prefix + '/share/thistle-tales/')
        update_desktop_file(self.prefix + '/share/thistle-tales/')
        DistUtilsExtra.auto.install_auto.run(self)
        update_data_path(self.prefix, previous_value)

       
##################################################################################
###################### YOU SHOULD MODIFY ONLY WHAT IS BELOW ######################
##################################################################################
DistUtilsExtra.auto.setup(
    name='thistle-tales',
    version='0.1',
    license='GPL-3',
    author='Giannis Tsagatakis (talos)',
    author_email='jtsagata@jmail.com',
    ext_modules = [ c_hacks_extension ],
    description='THistle tools for atitd atitd',
    long_description='Tools for working with thistles in game atitd.',
    url='https://launchpad.net/thistle-tales',
    cmdclass={'install': InstallAndUpdateDataDirectory}
    )

